package sklep.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import sklep.basket.Basket;
import sklep.model.Product;
import sklep.photo.PhotoUtil;
import sklep.repository.ProductRepository;

import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;

@Controller
@RequestMapping("/products")
public class ProductController {
    @Autowired
    private ProductRepository productRepository;

    @Autowired
    private PhotoUtil photoUtil;

    @GetMapping
    public String odczytajProdukty(Model model) {
        List<Product> products = productRepository.findAll(Sort.by("id"));
        model.addAttribute("products", products);
        return "products";
    }

    @GetMapping("/{id}")
    public String odczytajJedenProdukt(@PathVariable int id, Model model) {
        Optional<Product> product = productRepository.findById(id);
        if(product.isPresent()) {
            model.addAttribute("product", product.get());
            return "product";
        } else {
            model.addAttribute("product_id", id);
            return "missing_product";
        }
    }

    @GetMapping("/szukaj")
    public String szukaj(Model model,
                         @RequestParam(required=false) String name,
                         @RequestParam(required=false) BigDecimal min,
                         @RequestParam(required=false) BigDecimal max) {
        List<Product> products = List.of();
        if(name != null && !name.isEmpty() && min == null && max == null) {
            products = productRepository.findByProductNameContainsIgnoringCase(name);
        } else if ((name == null || name.isEmpty()) && (min != null || max != null)) {
            if(min == null) {
                min = BigDecimal.ZERO;
            }
            if(max == null) {
                max = BigDecimal.valueOf(1000_000_000);
            }
            products = productRepository.findByPriceBetween(min, max);
        } else if (name != null && !name.isEmpty()) {
            if(min == null) {
                min = BigDecimal.ZERO;
            }
            if(max == null) {
                max = BigDecimal.valueOf(1000_000_000);
            }
            products = productRepository.findByProductNameContainingIgnoringCaseAndPriceBetween(name, min, max);
        }
        model.addAttribute("products", products);
        return "wyszukiwarka2";
    }

    @GetMapping("/{id}/add-to-basket")
    public String addToBasket(@PathVariable("id") Integer productId,
                              @SessionAttribute Basket basket) {
        Optional<Product> product = productRepository.findById(productId);
        if(product.isPresent()) {
            basket.addProduct(product.get());
        } else {
            System.err.println("Nieznany produkt dodawany do koszyka: " + productId);
        }
        return "redirect:/products";
    }

    @GetMapping(path="/{id}/photo", produces="image/jpeg")
    @ResponseBody
    public byte[] getPhoto(@PathVariable int id) {
        return photoUtil.readBytes(id);
    }
}
